#include "Python.h"
#include <unicode/convert.h>
#include <unicode/bidi.h>
#include <unicode/unistr.h>
#include <stdio.h>
#include <assert.h>

#define isheb(c) (((((unsigned char) c)>=224) && (((unsigned char) c)<=250)) ? 1 : 0)

char *_log2vis(char *str, int str_len, UBiDiLevel paraLevel)
{
    UErrorCode errorcode = U_ZERO_ERROR;
    UnicodeConverter conv("iso-8859-8", errorcode);
    assert(U_SUCCESS(errorcode));   
    UnicodeString ustr, resultString;
    BiDi bidi(8192, 100, errorcode);
    assert(U_SUCCESS(errorcode));
    char *result;
    UChar uresult[8192];
    UTextOffset uresultlen;
    
    /* Convert from iso-8859-8 to Unicode */
    conv.toUnicodeString(ustr, str, str_len, errorcode);
    assert(U_SUCCESS(errorcode));
    
    /* Convert from Logical to Visual */
    bidi.setPara(ustr.getUChars(), ustr.size(), paraLevel, (UBiDiLevel *)NULL, errorcode);
    assert(U_SUCCESS(errorcode));
    uresultlen = bidi.writeReordered(uresult, 8192, UBIDI_REMOVE_BIDI_CONTROLS, errorcode);
    assert(U_SUCCESS(errorcode));
    
    /* Convert from Unicode to iso-8859-8 */
    result = (char *) malloc(sizeof(char) * (uresultlen + 1));
    resultString = UnicodeString(uresult, uresultlen);
    conv.fromUnicodeString(result, uresultlen, resultString, errorcode);
    assert(U_SUCCESS(errorcode));
    result[uresultlen] = 0;
    return result;
}


char log2vis__doc__[] = "log2vis(line): Convert a line from logical to visual hebrew.";

static PyObject *log2vis_log2vis(PyObject *self, PyObject *args) 
{
    char *str, *result;
    int paraLevel, str_len, max_chars = 80; 
    PyObject *pyResult;
    
    if (!PyArg_ParseTuple(args, "s#", &str, &str_len)) {
        return NULL;
    }
    
    if isheb(str[0]) {
        paraLevel = 1;
    } else {
        paraLevel = 0;
    }
    
    /* Don't bother with algorithm for empty strings (core dumps) */
    if (str_len > 0) {
        result = _log2vis(str, str_len, paraLevel);
    } else {
        result = (char *)malloc(sizeof(char));
        result[0] = '\x0';
    }
    
    pyResult = Py_BuildValue("s", result);
    free(result);
    
    return pyResult;
}


static PyMethodDef log2visMethods[] = {
    {"log2vis", log2vis_log2vis, METH_VARARGS, log2vis__doc__},      
    {NULL,      NULL}        /* Sentinel */
};

extern "C" {
    void init_log2vis(void)
    {
        (void) Py_InitModule("_log2vis", log2visMethods);
    }
}
